@@ -43,6 +43,7 @@
 
 #include <algorithm>
 #include <typeinfo>
+#include "exception.h"
 
 namespace clx {
 	/* --------------------------------------------------------------------- */
@@ -82,7 +83,7 @@ namespace clx {
 			}
 		}
 		
-		virtual ~shared_ptr() { this->decrement(); }
+		virtual ~shared_ptr() throw() { this->decrement(); }
 		
 		// operator
 		shared_ptr& operator=(const shared_ptr& cp) {
