@@ -92,9 +92,9 @@ namespace clx {
 		typedef typename rfdset::const_iterator const_iterator;
 		typedef struct timeval timeval_type;
 		
-		basic_sockmanager() : rfds_(), interval_(-1.0) { this->reset(); }
+		basic_sockmanager() : rfds_(), stop_(false), interval_(-1.0) {}
 		
-		virtual ~basic_sockmanager() throw() { this->reset(); }
+		virtual ~basic_sockmanager() throw() {}
 		
 		template <class Handler>
 		bool add(rawsocket_type* s, Handler h) {
@@ -146,7 +146,7 @@ namespace clx {
 				else if (n == 0) break; // timeout
 				
 				for (iterator pos = rfds_.begin(); pos != rfds_.end(); ++pos) {
-					if (FD_ISSET(pos->first->socket(), &sysrfds)) {
+					if (pos->first->pending() > 0 || FD_ISSET(pos->first->socket(), &sysrfds)) {
 						if (!pos->second->run(pos->first.get(), *this)) {
 							rfds_.erase(pos++);
 						}
@@ -178,11 +178,11 @@ namespace clx {
 		
 		iterator begin() { return rfds_.begin(); }
 		iterator end() { return rfds_.end(); }
-		rawsocket_type* socket(iterator pos) { return &(*pos->first); }
+		rawsocket_type* socket(iterator pos) { return pos->first.get(); }
 		
 		const_iterator begin() const { return rfds_.begin(); }
 		const_iterator end() const { return rfds_.end(); }
-		const rawsocket_type* socket(const_iterator pos) const { return &(*pos->first); }
+		const rawsocket_type* socket(const_iterator pos) const { return pos->first.get(); }
 		
 	private:
 		rfdset rfds_;
