@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Wed 22 Oct 2008 14:50:00 JST
+ *  Last-modified: Thu 06 Nov 2008 20:59:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_TCP_H
@@ -50,9 +50,9 @@ namespace clx {
 		//  basic_socket
 		/* ----------------------------------------------------------------- */
 		template <int Family>
-		class basic_socket : public basic_rawsocket<SOCK_STREAM, Family> {
+		class basic_socket : public basic_rawsocket<SOCK_STREAM, Family, 0> {
 		public:
-			typedef basic_rawsocket<SOCK_STREAM, Family> rawsocket;
+			typedef basic_rawsocket<SOCK_STREAM, Family, 0> rawsocket;
 			typedef basic_sockaddress<Family, IPPROTO_TCP> sockaddress;
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
@@ -62,13 +62,6 @@ namespace clx {
 			basic_socket(const basic_socket& cp) :
 				rawsocket(cp), addr_(cp.addr_), connect_(cp.connect_) {}
 			
-			basic_socket& operator=(const basic_socket& cp) {
-				addr_ = cp.addr_;
-				connect_ = cp.connect_;
-				this->assign(static_cast<rawsocket>(cp));
-				return *this;
-			}
-			
 			// for client socket
 			explicit basic_socket(const char_type* host, int port) :
 				rawsocket(), addr_(host, port), connect_(false) {
@@ -92,6 +85,13 @@ namespace clx {
 				}
 			}
 			
+			basic_socket& operator=(const basic_socket& cp) {
+				addr_ = cp.addr_;
+				connect_ = cp.connect_;
+				this->assign(static_cast<rawsocket>(cp));
+				return *this;
+			}
+			
 			// for server socket
 			explicit basic_socket(socket_int s, const sockaddress& addr) :
 				rawsocket(s), addr_(addr), connect_(true) {}
@@ -145,12 +145,19 @@ namespace clx {
 		};
 		
 		/* ----------------------------------------------------------------- */
-		//  basic_acceptor
+		/*
+		 *  basic_acceptor
+		 *
+		 *  The basic_acceptor class creates a welcome socket which
+		 *  accepts/rejects the client connecting request. When accepting
+		 *  a new client request, the class generates the basic_socket
+		 *  instance for the accepted client.
+		 */
 		/* ----------------------------------------------------------------- */
 		template <int Family>
-		class basic_acceptor : public basic_rawsocket<SOCK_STREAM, Family> {
+		class basic_acceptor : public basic_rawsocket<SOCK_STREAM, Family, 0> {
 		public:
-			typedef basic_rawsocket<SOCK_STREAM, Family> rawsocket;
+			typedef basic_rawsocket<SOCK_STREAM, Family, 0> rawsocket;
 			typedef basic_sockaddress<Family, IPPROTO_TCP> sockaddress;
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
@@ -212,21 +219,21 @@ namespace clx {
 		/* ----------------------------------------------------------------- */
 		//  port
 		/* ----------------------------------------------------------------- */
-		int port(const char* service) {
+		inline int port(const char* service) {
 			return clx::port(service, "tcp");
 		}
 		
-		int port(const std::basic_string<char>& service) {
+		inline int port(const std::basic_string<char>& service) {
 			return clx::tcp::port(service.c_str());
 		}
 		
-		typedef basic_rawsocket<SOCK_STREAM, AF_INET> rawsocket;
+		typedef basic_rawsocket<SOCK_STREAM, AF_INET, 0> rawsocket;
 		typedef basic_socket<AF_INET> socket;
 		typedef basic_acceptor<AF_INET> acceptor;
 		typedef basic_sockaddress<AF_INET, IPPROTO_TCP> sockaddress;
 		typedef basic_sockbuf<socket> sockbuf;
 		typedef basic_sockstream<socket> sockstream;
-		typedef basic_sockmanager<SOCK_STREAM, AF_INET> sockmanager;
+		typedef basic_sockmanager<SOCK_STREAM, AF_INET, 0> sockmanager;
 	};
 }
 
