@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Thu 10 Jul 2008 08:38:23 JST
+ *  Last-modified: Sat 25 Oct 2008 05:35:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_UDP_H
@@ -38,6 +38,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <string>
+#include "resolver.h"
 #include "socket.h"
 #include "sockaddress.h"
 #include "sockstream.h"
@@ -49,20 +50,18 @@ namespace clx {
 		//  basic_socket
 		/* ----------------------------------------------------------------- */
 		template <int Family>
-		class basic_socket : public basic_rawsocket<SOCK_DGRAM, Family> {
+		class basic_socket : public basic_rawsocket<SOCK_DGRAM, Family, 0> {
 		public:
-			typedef basic_rawsocket<SOCK_DGRAM, Family> rawsocket;
+			typedef basic_rawsocket<SOCK_DGRAM, Family, 0> rawsocket;
 			typedef basic_sockaddress<Family, IPPROTO_UDP> sockaddress;
-			typedef CharT char_type;
-			typedef std::basic_string<CharT, Traits> string_type;
+			typedef char char_type;
+			typedef std::basic_string<char> string_type;
 			
 			// constructor and destructor
 			basic_socket() : rawsocket(), from_(), to_(), is_bind_(false) {}
 			
 			basic_socket(const basic_socket& cp) :
-				from_(cp.from_), to_(cp.to_), is_bind_(cp.is_bind_) {
-				this->assign(cp);
-			}
+				rawsocket(cp), from_(cp.from_), to_(cp.to_), is_bind_(cp.is_bind_) {}
 			
 			basic_socket& operator=(const basic_socket& cp) {
 				from_ = cp.from_;
@@ -76,40 +75,45 @@ namespace clx {
 			
 			explicit basic_socket(const char_type* host, int port) :
 				rawsocket(), from_(), to_(), is_bind_(false) {
-				this->connect(host, port);
+				try {
+					this->connect(host, port);
+				}
+				catch (socket_error& e) {
+					this->assign(rawsocket());
+					throw e;
+				}
 			}
 			
 			explicit basic_socket(const string_type& host, int port) :
 				rawsocket(), from_(), to_(), is_bind_(false) {
-				this->connect(host, port);
-			}
-			
-			explicit basic_socket(const char_type* host, const char_type* service) :
-				rawsocket(), from_(), to_(), is_bind_(false) {
-				this->connect(host, service);
-			}
-			
-			explicit basic_socket(const string_type& host, const string_type& service) :
-				rawsocket(), from_(), to_(), is_bind_(false) {
-				this->connect(host, service);
+				try {
+					this->connect(host, port);
+				}
+				catch (socket_error& e) {
+					this->assign(rawsocket());
+					throw e;
+				}
 			}
 			
 			explicit basic_socket(int port) :
-				rawsocket(), from_(), to_(), is_bind_(false) { 
-				this->bind(port);
-			}
-			
-			explicit basic_socket(const char_type* service) :
 				rawsocket(), from_(), to_(), is_bind_(false) {
-				this->bind(service);
+				try {
+					this->bind(port);
+				}
+				catch (socket_error& e) {
+					this->assign(rawsocket());
+					throw e;
+				}
 			}
 			
-			virtual ~basic_socket() {}
+			virtual ~basic_socket() throw() {}
 			
 			// operator
 			basic_socket& connect(const char_type* host, int port) {
+				if (this->is_open()) this->close();
 				to_.assign(host, port);
 				this->open();
+				if (!this->is_open()) throw socket_error("open");
 				return *this;
 			}
 			
@@ -117,29 +121,9 @@ namespace clx {
 				return this->connect(host.c_str(), port);
 			}
 			
-			basic_socket& connect(const char_type* host, const char_type* service) {
-				to_.assign(host, service);
-				this->open();
-				return *this;
-			}
-			
-			basic_socket& connect(const string_type& host, const string_type& service) {
-				return this->connect(host.c_str(), service.c_str());
-			}
-			
-			int bind(int port) {
+			basic_socket& bind(int port) {
 				to_.assign(port);
-				return this->bind();
-			}
-			
-			int bind(const char_type* service) {
-				to_.assign(service);
-				return this->bind();
-			}
-			
-			int bind(const string_type& service) {
-				to_.assign(service);
-				return this->bind();
+				return this->xbind();
 			}
 			
 			int send_to(const char_type* src, int n, const sockaddress& addr) {
@@ -179,7 +163,7 @@ namespace clx {
 				int len = sizeof(tmp);
 				std::memset(dest, 0, n);
 				
-				int ret = ::recvfrom(this->socket(), dest, n, 0, (struct sockaddr*)&tmp, &len);
+				int ret = ::recvfrom(this->socket(), dest, n, 0, (struct sockaddr*)&tmp, (socklen_type*)&len);
 				if (ret > 0) from_ = tmp;
 				return ret;
 			}
@@ -194,19 +178,37 @@ namespace clx {
 			sockaddress to_;
 			bool is_bind_;
 			
-			int bind() {
+			basic_socket& xbind() {
+				if (this->is_open()) this->close();
 				this->open();
-				if (::bind(this->socket(), (struct sockaddr*)to_.data(), to_.size()) == -1) return -1;
+				
+				if (!this->is_open()) throw socket_error("open");
+				if (::bind(this->socket(), (struct sockaddr*)to_.data(), to_.size()) == -1) {
+					throw socket_error("bind");
+				}
+				
 				is_bind_ = true;
-				return 0;
+				return *this;
 			}
 		};
 		
-		typedef basic_rawsocket<SOCK_DGRAM, AF_INET> rawsocket;
+		/* ----------------------------------------------------------------- */
+		//  port
+		/* ----------------------------------------------------------------- */
+		inline int port(const char* service) {
+			return clx::port(service, "udp");
+		}
+		
+		inline int port(const std::basic_string<char>& service) {
+			return clx::udp::port(service.c_str());
+		}
+		
+		typedef basic_rawsocket<SOCK_DGRAM, AF_INET, 0> rawsocket;
 		typedef basic_socket<AF_INET> socket;
 		typedef basic_sockaddress<AF_INET, IPPROTO_UDP> sockaddress;
+		typedef basic_sockbuf<socket> sockbuf;
 		typedef basic_sockstream<socket> sockstream;
-		typedef basic_sockmanager<SOCK_DGRAM, AF_INET> sockmanager;
+		typedef basic_sockmanager<SOCK_DGRAM, AF_INET, 0> sockmanager;
 	};
 }
 
